// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

const omnibusGlobalOmnibusId = '*';

/// Handler pool base template
///
/// The handler pool manages handlers and indexing.
abstract class _HandlerPool<T extends OmnibusHandler> {
  /// Handlers
  List<T> _handlers = new List<T>();

  /// Hander index
  Map<String, List<int>> _index = new Map<String, List<int>>();

  /// Insert new handler
  void insert(T handler) {
    if (handler != null) {
      _handlers.add(handler);
      _index.putIfAbsent(handler.omnibusId, () => new List<int>());
      _index[handler.omnibusId].add(_handlers.length - 1);
    }
  }
}

/// _HandlerPool for EventHandlers
class _EventHandlerPool extends _HandlerPool<OmnibusEventHandler> {
  /// Publish
  ///
  /// This method runs in the future and all handlers are executed
  /// asynchronously.
  Future<List<Null>> publish(OmnibusEvent event) async {
    var futures = new List<Future<Null>>();

    // Run handlers.
    if (_index.containsKey(event.omnibusId)) {
      for (var i in _index[event.omnibusId]) {
        futures.addAll(await _handlers[i].process(event));
      }
    }

    // Run global handlers.
    if (_index.containsKey(omnibusGlobalOmnibusId)) {
      for (var i in _index[omnibusGlobalOmnibusId]) {
        futures.addAll(await _handlers[i].process(event));
      }
    }

    return Future.wait(futures);
  }
}

/// _HandlerPool for RequestHandlers
class _RequestHandlerPool extends _HandlerPool<OmnibusRequestHandler> {
  /// Publish
  ///
  /// This method runs in the future and all handlers are executed
  /// asynchronously.
  Future<OmnibusResponses> publish(OmnibusRequest request) async {
    var responses = new List<Future<OmnibusResponse>>();

    // Collect responses.
    if (_index.containsKey(request.omnibusId)) {
      for (var i in _index[request.omnibusId]) {
        responses.addAll(await _handlers[i].process(request));
      }
    }

    // Collect global responses.
    if (_index.containsKey(omnibusGlobalOmnibusId)) {
      for (var i in _index[omnibusGlobalOmnibusId]) {
        responses.addAll(await _handlers[i].process(request));
      }
    }

    return new OmnibusResponses(responses);
  }
}
