// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.request_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';
import 'package:omnibus/patterns.dart';

import 'messages_lib.dart' as v0;

void main() {
  test('Run calculations using v0.CalculationRequest messages', () async {
    var bus = new Omnibus();

    // Bind calculation handlers.
    bus.bind(new v0.CalculationRequestHandler(calcAddHandler, new EmptyState(),
        op: v0.CalculationOperator.add));
    bus.bind(new v0.CalculationRequestHandler(
        calcSubtractHandler, new EmptyState(),
        op: v0.CalculationOperator.subtract));
    bus.bind(new v0.CalculationRequestHandler(
        calcMultiplyHandler, new EmptyState(),
        op: v0.CalculationOperator.multiply));
    bus.bind(new v0.CalculationRequestHandler(
        calcDivideHandler, new EmptyState(),
        op: v0.CalculationOperator.divide));

    // Request addition, raw retrieval style.
    List<OmnibusResponse> responses = await (await bus.publishRequest(
            new v0.CalculationRequest(1.0, 1.0, v0.CalculationOperator.add)))
        .onComplete();

    expect(responses.length, equals(1));
    responses
        .where((OmnibusResponse response) =>
            response.omnibusId == v0.CalculationResponse.id)
        .forEach((OmnibusResponse response) {
      var r = response is v0.CalculationResponse
          ? response
          : new v0.CalculationResponse.fromPds(response.toPds());
      expect(r.result.result, equals(2.0));
    });

    // Request addition.
    await bus
        .publishRequest(
            new v0.CalculationRequest(1.0, 1.0, v0.CalculationOperator.add))
        .then((OmnibusResponses responses) async {
      expect(responses.count, equals(1));
      var calculations = v0.streamCalculationResponse(responses);
      await for (var calculation in calculations) {
        expect(calculation.result.result, equals(2.0));
      }
    });

    // Request division.
    var allResponses = await bus.publishRequest(
        new v0.CalculationRequest(12.0, 8.0, v0.CalculationOperator.divide));

    // Await all responses to check if they are still handled correctly.
    await allResponses.onComplete();

    await for (var calculation in v0.streamCalculationResponse(allResponses)) {
      // 12.0 / 8.0 = 1.5
      expect(calculation.result.result, equals(1.5));
    }

    // Request multiplication.
    var request =
        new v0.CalculationRequest(2.0, 8.0, v0.CalculationOperator.multiply);

    await for (var calculation
        in v0.streamCalculationResponse(await bus.request(request))) {
      // 2.0 * 8.0 = 16.0
      expect(calculation.result.result, equals(16.0));
    }
  });
}

/// Test with raw response data.
class RawCalculationResponse extends OmnibusResponse {
  /// omnibusId
  static const id = 'omnibuild.v0.CalculationResponse';

  /// Calculation result
  final v0.CalculationResult result;

  /// Constructor
  RawCalculationResponse(this.result) : super(id);

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'result': result.toPds()};
  }
}

Future<RawCalculationResponse> calcAddHandler(
    v0.CalculationRequest r, StateData state) async {
  return new RawCalculationResponse(new v0.CalculationResult(r.a + r.b));
}

Future<v0.CalculationResponse> calcSubtractHandler(
    v0.CalculationRequest r, StateData state) async {
  return new v0.CalculationResponse(new v0.CalculationResult(r.a - r.b));
}

Future<v0.CalculationResponse> calcMultiplyHandler(
    v0.CalculationRequest r, StateData state) async {
  return new v0.CalculationResponse(new v0.CalculationResult(r.a * r.b));
}

Future<v0.CalculationResponse> calcDivideHandler(
    v0.CalculationRequest r, StateData state) async {
  // Delay by 1 second to run through future callbacks in Responses.forEach.
  return new Future<OmnibusResponse>.delayed(new Duration(seconds: 1),
      () => new v0.CalculationResponse(new v0.CalculationResult(r.a / r.b)));
}
